@@ -49,10 +49,7 @@ class RecordSpecification:
                 control_table_keys = []  # single row records don't need to be keyed
         if isinstance(control_table_keys, str):
             control_table_keys = [control_table_keys]
-        if self.control_table.shape[0] == 1:
-            if len(control_table_keys) > 0:
-                raise ValueError("signle-row records must not have control table keys")
-        else:
+        if strict and (self.control_table.shape[0] > 1):
             if len(control_table_keys) <= 0:
                 raise ValueError("multi-row records must have at least one control table key")
         self.control_table_keys = [k for k in control_table_keys]
@@ -158,7 +155,7 @@ class RecordSpecification:
 
         return RecordMap(blocks_in=self)
 
-    def map_from_rows(self, *, strict=False):
+    def map_from_rows(self):
         """
         Build a RecordMap mapping this RecordSpecification from rowrecs
 
@@ -499,7 +496,7 @@ class RecordMap:
                 + "\n"
             )
             return s
-        else:
+        if self.blocks_out is not None:
             s = (
                 "Transform row records of the form:\n"
                 + "  record_keys: "
@@ -512,6 +509,7 @@ class RecordMap:
                 + str(self.blocks_out)
             )
             return s
+        raise ValueError("should not be reached")
 
     def __repr__(self):
         s = (
