@@ -46,12 +46,23 @@ def pretty_format_python(python_str: str, *, black_mode=None) -> str:
     return formatted_python
 
 
+def _assert_tables_defs_consistent(tm1: Dict, tm2: Dict):
+    common_keys = set(tm1.keys()).intersection(tm2.keys())
+    for k in common_keys:
+        t1 = tm1[k]
+        t2 = tm2[k]
+        if not t1.same_table(t2):
+            raise ValueError(
+                "Table " + k + " has two incompatible representations"
+            )
+
+
 class ViewRepresentation(OperatorPlatform, ABC):
     """Structure to represent the columns of a query or a table.
        Abstract base class."""
 
     column_names: List[str]
-    column_set: data_algebra.OrderedSet.OrderedSet
+    column_set: data_algebra.OrderedSet.OrderedSet[str]
     column_types: Optional[Dict[str, type]]
     sources: List[
         "ViewRepresentation"
@@ -68,23 +79,15 @@ class ViewRepresentation(OperatorPlatform, ABC):
         if isinstance(column_names, str):
             column_names = [column_names]
         else:
-            column_names = [v for v in column_names]  # make sure a list and a disjoint copy
-        assert isinstance(column_names, list)
-        assert all([isinstance(v, str) for v in column_names])
+            column_names = list(column_names)  # make sure a list and a disjoint copy
         self.column_names = column_names
-        for ci in self.column_names:
-            assert isinstance(ci, str)
-        if len(self.column_names) < 1:
-            raise ValueError("no column names")
+        assert len(self.column_names) > 0
+        for v in self.column_names:
+            assert isinstance(v, str)
         self.column_set = data_algebra.OrderedSet.OrderedSet()
         for c in self.column_names:
             self.column_set.add(c)
-        if not len(self.column_names) == len(self.column_set):
-            raise ValueError("duplicate column name(s)")
-        column_dict = {
-            ci: data_algebra.expr_rep.ColumnReference(self, ci)
-            for ci in self.column_names
-        }
+        assert len(self.column_names) == len(self.column_set)
         if sources is None:
             sources = []
         for si in sources:
@@ -93,9 +96,16 @@ class ViewRepresentation(OperatorPlatform, ABC):
         self.column_types = None
         if column_types is not None:
             self.column_types = column_types.copy()
-        OperatorPlatform.__init__(
-            self, node_name=node_name, column_map=collections.OrderedDict(**column_dict)
-        )
+        OperatorPlatform.__init__(self, node_name=node_name)
+
+    def column_map(self) -> collections.OrderedDict:
+        """
+        Build a map of column names to ColumnReferences
+        """
+        res = collections.OrderedDict()
+        for ci in self.column_names:
+            res[ci] = data_algebra.expr_rep.ColumnReference(self, ci)
+        return res
 
     def merged_rep_id(self) -> str:
         return "ops+ " + str(id(self))
@@ -126,20 +136,26 @@ class ViewRepresentation(OperatorPlatform, ABC):
     def get_tables(self):
         """Get a dictionary of all tables used in an operator DAG,
         raise an exception if the values are not consistent."""
-        tables = {}
-        for i in range(len(self.sources)):
-            s = self.sources[i]
-            ti = s.get_tables()
-            for (k, v) in ti.items():
-                assert isinstance(v, TableDescription)
-                assert v.node_name == "TableDescription"
+
+        tables = dict()
+        # eliminate recursions by stepping through sources
+        visit_stack = list()
+        visit_stack.append(self)
+        while len(visit_stack) > 0:
+            cursor = visit_stack.pop()
+            if isinstance(cursor, TableDescription):
+                k = cursor.key
+                v = cursor
                 if k in tables.keys():
                     if not v.same_table(tables[k]):
                         raise ValueError(
-                            "Table " + k + " has two different representation objects"
+                            "Table " + k + " has two incompatible representations"
                         )
                 else:
                     tables[k] = v
+            else:
+                for s in cursor.sources:
+                    visit_stack.append(s)
         return tables
 
     def columns_used_from_sources(self, using=None):
@@ -247,8 +263,6 @@ class ViewRepresentation(OperatorPlatform, ABC):
             return False
         if self.column_names != other.column_names:
             return False
-        if self.column_map != other.column_map:
-            return False
         if len(self.sources) != len(other.sources):
             return False
         if not self._equiv_nodes(other):
@@ -455,7 +469,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
             order_by = [order_by]
         if isinstance(reverse, str):
             reverse = [reverse]
-        parsed_ops = data_algebra.expr_parse.parse_assignments_in_context(ops, self)
+        parsed_ops = data_algebra.expr_parse.parse_assignments_in_context(ops=ops, view=self)
         return self.extend_parsed(
             parsed_ops=parsed_ops,
             partition_by=partition_by,
@@ -482,7 +496,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
             group_by = []
         if ((ops is None) or (len(ops) < 1)) and (len(group_by) < 1):
             raise ValueError("must have ops or group_by")
-        parsed_ops = data_algebra.expr_parse.parse_assignments_in_context(ops, self)
+        parsed_ops = data_algebra.expr_parse.parse_assignments_in_context(ops=ops, view=self)
         return self.project_parsed(parsed_ops=parsed_ops, group_by=group_by)
 
     def natural_join(self, b, *, by, jointype, check_all_common_keys_in_by=False):
@@ -545,7 +559,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
         assert isinstance(expr, (str, data_algebra.expr_rep.PreTerm))
         if self.is_trivial_when_intermediate():
             return self.sources[0].select_rows(expr)
-        ops = data_algebra.expr_parse.parse_assignments_in_context({"expr": expr}, self)
+        ops = data_algebra.expr_parse.parse_assignments_in_context(ops={"expr": expr}, view=self)
 
         def r_walk_expr(opv):
             if not isinstance(opv, data_algebra.expr_rep.Expression):
@@ -921,8 +935,6 @@ class ExtendNode(ViewRepresentation):
         windowed_situation = data_algebra.expr_rep.implies_windowed(parsed_ops)
         ordered_windowed_situation = False
         self.ops = parsed_ops
-        self.cols_used_in_calc = data_algebra.expr_rep.get_columns_used(parsed_ops)
-        self.cols_produced_in_calc = [k for k in parsed_ops.keys()]
         if partition_by is None:
             partition_by = []
         if isinstance(partition_by, numbers.Number):
@@ -1702,6 +1714,7 @@ class NaturalJoinNode(ViewRepresentation):
         # check set of tables is consistent in both sub-dags
         a_tables = a.get_tables()
         b_tables = b.get_tables()
+        _assert_tables_defs_consistent(a_tables, b_tables)
         if by is None:
             raise ValueError(
                 "Must specify by in natural joins ([] for empty conditions)"
@@ -1746,7 +1759,6 @@ class NaturalJoinNode(ViewRepresentation):
         self.jointype = data_algebra.expr_rep.standardize_join_type(jointype)
         if (self.jointype == "CROSS") and (len(self.by) != 0):
             raise ValueError("CROSS joins must have an empty 'by' list")
-        self.get_tables()  # causes a throw if left and right table descriptions are inconsistent
 
     def apply_to(self, a, *, target_table_key=None):
         new_sources = [
@@ -1820,6 +1832,7 @@ class ConcatRowsNode(ViewRepresentation):
         assert isinstance(b, ViewRepresentation)
         a_tables = a.get_tables()
         b_tables = b.get_tables()
+        _assert_tables_defs_consistent(a_tables, b_tables)
         common_keys = set(a_tables.keys()).intersection(b_tables.keys())
         for k in common_keys:
             if not a_tables[k].same_table(b_tables[k]):
@@ -1855,7 +1868,6 @@ class ConcatRowsNode(ViewRepresentation):
         self.id_column = id_column
         self.a_name = a_name
         self.b_name = b_name
-        self.get_tables()  # causes a throw if left and right table descriptions are inconsistent
 
     def apply_to(self, a, *, target_table_key=None):
         new_sources = [
@@ -2058,6 +2070,9 @@ class SQLNode(ViewRepresentation):
             return False
         return True
 
+    def get_tables(self):
+        return dict()
+
     def columns_used_from_sources(self, using=None):
         return []
 
