@@ -1,7 +1,14 @@
+"""
+Utility to merge extend operations, when appropriate.
+"""
+
 import data_algebra.expr_rep
 
 
 def try_to_merge_ops(ops1, ops2):
+    """
+    Try to merge two extends into one. Return merged op, or None if not possible.
+    """
     ops1_columns_used = set(data_algebra.expr_rep.get_columns_used(ops1))
     ops1_columns_produced = set([k for k in ops1.keys()])
     ops2_columns_used = set(data_algebra.expr_rep.get_columns_used(ops2))
@@ -13,11 +20,9 @@ def try_to_merge_ops(ops1, ops2):
         ops1_common_columns_used = set(
             data_algebra.expr_rep.get_columns_used(ops1_common)
         )
-        ops1_common_columns_produced = set([k for k in ops1_common.keys()])
         ops2_common_columns_used = set(
             data_algebra.expr_rep.get_columns_used(ops2_common)
         )
-        ops2_common_columns_produced = set([k for k in ops2_common.keys()])
         if len(ops1_common_columns_used.intersection(ops2_columns_produced)) > 0:
             return None
         if len(ops1_common_columns_used.intersection(ops1_columns_produced)) > 0:
