@@ -55,7 +55,7 @@ class SQLFormatOptions(SimpleNamespace):
         p.text(str(self))
 
 
-def _list_join_expecting_list(joiner: str, str_list: List[str]):
+def _list_join_expecting_list(joiner: str, str_list: List[str]) -> List[str]:
     assert isinstance(joiner, str)
     assert isinstance(str_list, list)
     assert all([isinstance(vi, str) for vi in str_list])
@@ -63,7 +63,7 @@ def _list_join_expecting_list(joiner: str, str_list: List[str]):
     return [' ' + str_list[i] + (joiner if i < (n - 1) else '') for i in range(n)]
 
 
-def _clean_annotation(annotation: Optional[str]):
+def _clean_annotation(annotation: Optional[str]) -> Optional[str]:
     assert isinstance(annotation, (str, type(None)))
     if annotation is None:
         return annotation
@@ -117,11 +117,11 @@ def _db_is_bad_expr(dbmodel, expression):
         + " IS NULL OR "
         + subexpr
         + " >= "
-        + dbmodel.quote_literal("+infinity")
+        + dbmodel.value_to_sql("+infinity")
         + " OR "
         + subexpr
         + " <= "
-        + dbmodel.quote_literal("-infinity")
+        + dbmodel.value_to_sql("-infinity")
         + " OR ("
         + subexpr
         + " != 0 AND "
@@ -512,21 +512,21 @@ class DBModel:
     def __init__(
         self,
         *,
-        identifier_quote='"',
-        string_quote="'",
+        identifier_quote: str = '"',
+        string_quote: str = "'",
         sql_formatters=None,
         op_replacements=None,
         local_data_model=None,
-        on_start="",
-        on_end="",
-        on_joiner="AND",
-        drop_text="DROP TABLE",
-        string_type="VARCHAR",
-        supports_with=True,
-        allow_extend_merges=True,
+        on_start: str = "",
+        on_end: str = "",
+        on_joiner: str = "AND",
+        drop_text: str = "DROP TABLE",
+        string_type: str = "VARCHAR",
+        supports_with: bool = True,
+        allow_extend_merges: bool = True,
         default_SQL_format_options=None,
-        union_all_term_start="(",
-        union_all_term_end=")",
+        union_all_term_start: str = "(",
+        union_all_term_end: str = ")",
     ):
         if local_data_model is None:
             local_data_model = data_algebra.default_data_model
@@ -596,7 +596,8 @@ class DBModel:
         r = r.reset_index(drop=True)
         return r
 
-    def table_exists(self, conn, table_name):
+    def table_exists(self, conn, table_name: str) -> bool:
+        assert isinstance(table_name, str)
         q_table_name = self.quote_table_name(table_name)
         # noinspection PyBroadException
         table_exists = True
@@ -606,15 +607,15 @@ class DBModel:
             table_exists = False
         return table_exists
 
-    def drop_table(self, conn, table_name, *, check=True):
+    def drop_table(self, conn, table_name: str, *, check: bool = True) -> None:
         if (not check) or self.table_exists(conn, table_name):
             q_table_name = self.quote_table_name(table_name)
             self.execute(conn, self.drop_text + " " + q_table_name)
 
     # noinspection PyMethodMayBeStatic,SqlNoDataSourceInspection
     def insert_table(
-        self, conn, d, table_name, *, qualifiers=None, allow_overwrite=False
-    ):
+        self, conn, d, table_name: str, *, qualifiers=None, allow_overwrite=False
+    ) -> None:
         """
         Insert a table.
 
@@ -634,10 +635,11 @@ class DBModel:
                 self.drop_table(conn, table_name, check=False)
         # Note: the Pandas to_sql() method appears to have SQLite hard-wired into it
         # it refers to sqlite_master
+        # this behavior seems to change if sqlalchemy is active
         d.to_sql(name=table_name, con=conn, index=False)
 
     # noinspection PyMethodMayBeStatic,PyUnusedLocal
-    def read_table(self, conn, table_name, *, qualifiers=None, limit=None):
+    def read_table(self, conn, table_name: str, *, qualifiers=None, limit=None):
         assert isinstance(table_name, str)
         q_table_name = self.quote_table_name(table_name)
         sql = "SELECT * FROM " + q_table_name
@@ -654,7 +656,7 @@ class DBModel:
             conn=conn, table_name=table.table_name, qualifiers=table.qualifiers
         )
 
-    def quote_identifier(self, identifier):
+    def quote_identifier(self, identifier: str) -> str:
         assert isinstance(identifier, str)
         if self.identifier_quote in identifier:
             raise ValueError(
@@ -662,7 +664,7 @@ class DBModel:
             )
         return self.identifier_quote + identifier + self.identifier_quote
 
-    def quote_table_name(self, table_description):
+    def quote_table_name(self, table_description) -> str:
         if not isinstance(table_description, str):
             try:
                 if table_description.node_name == "TableDescription":
@@ -677,7 +679,7 @@ class DBModel:
                 )
         return self.quote_identifier(table_description)
 
-    def quote_string(self, string):
+    def quote_string(self, string: str) -> str:
         assert isinstance(string, str)
         # replace all string with doubled string quotes
         return (
@@ -686,10 +688,7 @@ class DBModel:
             + self.string_quote
         )
 
-    def quote_literal(self, val):
-        return self.quote_string(str(val))
-
-    def value_to_sql(self, v):
+    def value_to_sql(self, v) -> str:
         if v is None:
             return "NULL"
         if isinstance(v, data_algebra.expr_rep.ListTerm):
@@ -713,7 +712,7 @@ class DBModel:
             return "(" + ", ".join([self.value_to_sql(vi) for vi in v]) + ")"
         return str(v)
 
-    def table_values_to_sql_str_list(self, v) -> str:
+    def table_values_to_sql_str_list(self, v, *, result_name: str = 'table_values') -> List[str]:
         assert v is not None
         m = v.shape[0]
         assert m > 0
@@ -738,11 +737,11 @@ class DBModel:
         sql = (
             ["SELECT", " *", "FROM ("]
             + ["    " + ("" if (i < 1) else "UNION ALL ") + q_row(i) for i in range(m)]
-            + [f') {qi("table_values")}']
+            + [f') {qi(result_name)}']
         )
         return sql
 
-    def expr_to_sql(self, expression, *, want_inline_parens=False):
+    def expr_to_sql(self, expression, *, want_inline_parens: bool = False) -> str:
         if isinstance(expression, str):
             return expression
         assert isinstance(expression, data_algebra.expr_rep.PreTerm)
@@ -780,7 +779,7 @@ class DBModel:
             return self.value_to_sql(expression.value)
         raise TypeError("unexpected type: " + str(type(expression)))
 
-    def _indent_and_sep_terms(self, terms, *, sep=",", sql_format_options=None):
+    def _indent_and_sep_terms(self, terms, *, sep: str = ",", sql_format_options=None) -> List[str]:
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
         n = len(terms)
@@ -801,7 +800,7 @@ class DBModel:
             for i in range(n)
         ]
 
-    def table_def_to_sql(
+    def table_def_to_near_sql(
         self, table_def, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
         if table_def.node_name != "TableDescription":
@@ -829,17 +828,17 @@ class DBModel:
             # need a non-trivial select here
             terms = OrderedDict()
             for k in using:
-                terms[k] = k
+                terms[k] = k  # these get quoted later
             view_name = "table_reference_" + str(temp_id_source[0])
             temp_id_source[0] = temp_id_source[0] + 1
             near_sql = data_algebra.near_sql.NearSQLUnaryStep(
-                terms=terms,  # TODO: implement pruning
+                terms=terms,
                 quoted_query_name=self.quote_identifier(view_name),
                 sub_sql=subsql.to_bound_near_sql(columns=using),
             )
         return near_sql
 
-    def extend_to_sql(
+    def extend_to_near_sql(
         self, extend_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
         if extend_node.node_name != "ExtendNode":
@@ -973,9 +972,9 @@ class DBModel:
         )
         return near_sql
 
-    def project_to_sql(
+    def project_to_near_sql(
         self, project_node, *, using=None, temp_id_source=None, sql_format_options=None
-    ):
+    ) -> data_algebra.near_sql.NearSQL:
         if project_node.node_name != "ProjectNode":
             raise TypeError(
                 "Expected project_node to be a data_algebra.data_ops.ProjectNode)"
@@ -1010,7 +1009,7 @@ class DBModel:
         )
         return near_sql
 
-    def select_rows_to_sql(
+    def select_rows_to_near_sql(
         self,
         select_rows_node,
         *,
@@ -1050,7 +1049,7 @@ class DBModel:
         )
         return near_sql
 
-    def select_columns_to_sql(
+    def select_columns_to_near_sql(
         self,
         select_columns_node,
         *,
@@ -1060,7 +1059,7 @@ class DBModel:
     ) -> data_algebra.near_sql.NearSQL:
         if select_columns_node.node_name != "SelectColumnsNode":
             raise TypeError(
-                "Expected select_columns_to_sql to be a data_algebra.data_ops.SelectColumnsNode)"
+                "Expected select_columns_to_near_sql to be a data_algebra.data_ops.SelectColumnsNode)"
             )
         if temp_id_source is None:
             temp_id_source = [0]
@@ -1084,7 +1083,7 @@ class DBModel:
             subsql.terms = []
         return subsql
 
-    def drop_columns_to_sql(
+    def drop_columns_to_near_sql(
         self,
         drop_columns_node,
         *,
@@ -1112,7 +1111,7 @@ class DBModel:
         }
         return subsql
 
-    def order_to_sql(
+    def order_to_near_sql(
         self, order_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
         if order_node.node_name != "OrderRowsNode":
@@ -1162,7 +1161,7 @@ class DBModel:
         )
         return near_sql
 
-    def rename_to_sql(
+    def rename_to_near_sql(
         self, rename_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
         if rename_node.node_name != "RenameColumnsNode":
@@ -1242,7 +1241,7 @@ class DBModel:
         )
         return using_left, sql_left, using_right, sql_right
 
-    def natural_join_to_sql(
+    def natural_join_to_near_sql(
         self, join_node, *, using=None, temp_id_source=None, sql_format_options=None, left_is_first=True
     ) -> data_algebra.near_sql.NearSQL:
         if temp_id_source is None:
@@ -1300,7 +1299,7 @@ class DBModel:
         )
         return near_sql
 
-    def concat_rows_to_sql(
+    def concat_rows_to_near_sql(
         self, concat_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
         if concat_node.node_name != "ConcatRowsNode":
@@ -1421,8 +1420,7 @@ class DBModel:
         sql_str_list = [v for v in sql_str_list if len(v) > 0]
         return "\n".join(sql_str_list) + "\n"
 
-    def row_recs_to_blocks_query_str_list_pair(
-        self, record_spec) -> Tuple[List[str], List[str]]:
+    def row_recs_to_blocks_query_str_list_pair(self, record_spec) -> Tuple[List[str], List[str]]:
         control_value_cols = [
             c
             for c in record_spec.control_table.columns
@@ -1487,9 +1485,7 @@ class DBModel:
         return sql_prefix, sql_suffix
 
     # noinspection PyUnusedLocal
-    def blocks_to_row_recs_query_str_list_pair(
-        self,
-        record_spec) -> Tuple[List[str], List[str]]:
+    def blocks_to_row_recs_query_str_list_pair(self, record_spec) -> Tuple[List[str], List[str]]:
         assert record_spec.control_table.shape[0] >= 1
         col_stmts = []
         for c in record_spec.record_keys:
@@ -1499,10 +1495,11 @@ class DBModel:
         if record_spec.control_table.shape[0] == 1:
             # special case with one row control table (rowrec)
             for cc in record_spec.control_table.columns:
-                cc0 = record_spec.control_table[cc][0]
-                col_stmts.append(
-                    " " + self.quote_identifier(cc) + " AS " + self.quote_identifier(cc0)
-                )
+                if cc not in record_spec.control_table_keys:
+                    cc0 = record_spec.control_table[cc][0]
+                    col_stmts.append(
+                        " " + self.quote_identifier(cc) + " AS " + self.quote_identifier(cc0)
+                    )
             sql_prefix = (
                     _list_join_expecting_list(',', col_stmts)
                     + ["FROM ( SELECT * FROM "]
@@ -1715,25 +1712,26 @@ class DBModel:
         self,
         near_sql,
         *,
-        columns=None,
-        force_sql=False,
-        constants=None,
         sql_format_options=None,
+        add_select=True,
     ) -> List[str]:
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLRawQStep)
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
         assert isinstance(sql_format_options, SQLFormatOptions)
-        sql = (
-            ["SELECT"]
-            + [' ' + v for v in near_sql.prefix]
-            + [
+        sql = []
+        if near_sql.annotation is not None:
+            sql = sql + ["-- " + _clean_annotation(near_sql.annotation)]
+        if add_select:
+            sql = sql + ["SELECT"]
+        sql = sql + [' ' + v for v in near_sql.prefix]
+        if near_sql.sub_sql is not None:
+            sql = sql + [
                 sql_format_options.sql_indent + si
                 for si in near_sql.sub_sql.convert_subsql(
                     db_model=self, sql_format_options=sql_format_options
                 )
             ]
-        )
         if (near_sql.suffix is not None) and (len(near_sql.suffix) > 0):
             sql = sql + [' ' + v for v in near_sql.suffix]
         return sql
@@ -1815,7 +1813,7 @@ class DBModel:
 
 
 class DBHandle:
-    def __init__(self, *, db_model, conn):
+    def __init__(self, *, db_model: DBModel, conn):
         assert isinstance(db_model, DBModel)
         self.db_model = db_model
         self.conn = conn
@@ -1830,7 +1828,7 @@ class DBHandle:
     def read_query(self, q):
         return self.db_model.read_query(conn=self.conn, q=q)
 
-    def describe_table(self, table_name, *, qualifiers=None, row_limit=7):
+    def describe_table(self, table_name: str, *, qualifiers=None, row_limit: Optional[int] = 7):
         head = self.read_query(
             q="SELECT * FROM "
             + self.db_model.quote_table_name(table_name)
@@ -1841,32 +1839,13 @@ class DBHandle:
             head, table_name=table_name, qualifiers=qualifiers, row_limit=row_limit
         )
 
-    def to_pandas(self, handle, *, data_map=None):
-        if isinstance(handle, data_algebra.data_ops.TableDescription):
-            handle = handle.table_name
-        assert isinstance(handle, str)
-        if data_map is not None:
-            if handle not in data_map:
-                return ValueError("Expected handle to be a data_map key " + handle)
-            assert isinstance(data_map[handle], data_algebra.data_ops.TableDescription)
-            if data_map[handle].table_name != handle:
-                raise ValueError(
-                    "data_map["
-                    + handle
-                    + "].table_name == "
-                    + data_map[handle].table_name
-                    + ", not "
-                    + handle
-                )
-        return self.db_model.read_table(self.conn, handle)
-
-    def execute(self, q):
+    def execute(self, q) -> None:
         self.db_model.execute(conn=self.conn, q=q)
 
-    def drop_table(self, table_name):
+    def drop_table(self, table_name: str) -> None:
         self.db_model.drop_table(self.conn, table_name)
 
-    def insert_table(self, d, *, table_name, allow_overwrite=False):
+    def insert_table(self, d, *, table_name: str, allow_overwrite: bool = False):
         self.db_model.insert_table(
             conn=self.conn, d=d, table_name=table_name, allow_overwrite=allow_overwrite
         )
@@ -1877,12 +1856,12 @@ class DBHandle:
     ) -> str:
         return self.db_model.to_sql(ops=ops, sql_format_options=sql_format_options,)
 
-    def query_to_csv(self, q, *, res_name):
+    def query_to_csv(self, q, *, res_name: str) -> None:
         d = self.read_query(q)
         d.to_csv(res_name, index=False)
 
-    def table_values_to_sql_str_list(self, v) -> str:
-        return self.db_model.table_values_to_sql_str_list(v)
+    def table_values_to_sql_str_list(self, v, *, result_name: str = 'table_values') -> List[str]:
+        return self.db_model.table_values_to_sql_str_list(v, result_name=result_name)
 
     def __str__(self):
         return (
@@ -1898,7 +1877,7 @@ class DBHandle:
     def __repr__(self):
         return self.__str__()
 
-    def close(self):
+    def close(self) -> None:
         if self.conn is not None:
             try:
                 self.conn.close()
