@@ -1,3 +1,7 @@
+"""
+Representation for operations that are nearly translated into SQL.
+"""
+
 from abc import ABC
 from typing import List, Optional, Tuple
 
@@ -347,15 +351,17 @@ class NearSQLRawQStep(NearSQL):
         *,
         prefix: List[str],
         quoted_query_name: str,
-        sub_sql: NearSQLContainer,
+        sub_sql: Optional[NearSQLContainer],
         suffix: Optional[List[str]] = None,
         annotation: Optional[str] = None,
+        add_select: bool = True,
     ):
+        assert isinstance(add_select, bool)
         assert isinstance(prefix, list)
         assert len(prefix) > 0
         assert all([isinstance(v, str) for v in prefix])
         assert isinstance(quoted_query_name, str)
-        assert isinstance(sub_sql, NearSQLContainer)
+        assert isinstance(sub_sql, (NearSQLContainer, type(None)))
         assert isinstance(suffix, (list, type(None)))
         if suffix is not None:
             assert all([isinstance(v, str) for v in suffix])
@@ -369,6 +375,7 @@ class NearSQLRawQStep(NearSQL):
         self.prefix = prefix
         self.sub_sql = sub_sql
         self.suffix = suffix
+        self.add_select = add_select
 
     def to_sql_str_list(
         self,
@@ -381,12 +388,14 @@ class NearSQLRawQStep(NearSQL):
     ) -> List[str]:
         return db_model.nearsqlrawq_to_sql_str_list_(
             near_sql=self,
-            columns=columns,
-            constants=constants,
             sql_format_options=sql_format_options,
+            add_select=self.add_select,
         )
 
     def to_with_form(self) -> SQLWithList:
+        if self.sub_sql is None:
+            # no sub-steps
+            return SQLWithList(last_step=self, previous_steps=[])
         if self.sub_sql.near_sql.is_table:
             # table references don't need to be re-encoded
             return SQLWithList(last_step=self, previous_steps=[])
@@ -398,5 +407,6 @@ class NearSQLRawQStep(NearSQL):
             sub_sql=stub,
             suffix=self.suffix,
             annotation=self.annotation,
+            add_select=self.add_select,
         )
         return SQLWithList(last_step=stubbed_step, previous_steps=sequence)
